#!perl -w

use XML::Parser;
use File::DosGlob;
use Date::Manip;
do "subs.pl";

Date_Init("Language=French","DateFormat=nonUS","TZ=MET");

$parser = new XML::Parser(NoExpand => 1);

%index = ();
$debug = 0;
$docnum = 0;
$basename = "";
$current_ana = "";
$reg = "";
$last_line = "";
$date_crime = "";
$date_signature = "";
%notaires = ();
$num_notaire = 1;

sub handle_start {
    my ($expat, $element, %atts) = @_;

    if ($element eq "body") {
	$in_body = 1;
    } elsif (! $in_body) {
	return;
    } elsif (defined $atts{ana} &&
	     $atts{ana} =~ /(diplomatique-date-signature-chancellerie|diplomatique-notaire)/) {
	$current_ana = $1;
	if ($atts{ana} =~ /notaire (.*)$/) {
	    $notaire =$1;
	    if (! defined $notaires{$notaire}) {
		$notaires{$notaire} = $num_notaire++;
	    }
	}
	print STDERR "Trouve '$current_ana' a $last_line\n" if $debug;
	$in_key = 1;
	$reg = "";
	$current_element = $element;
	if (defined $atts{reg}) {
	    $reg =  clean_lat1($atts{reg});
	    print STDERR "reg dans $element\n" if (! ($element eq "name" ||
						      $element eq "rs"));
	} elsif (defined $atts{value}) {
	    $reg =  clean_lat1($atts{value});
	}
    }

}

sub handle_end {
    my ($expat, $element) = @_;
    if ($in_key) {
	if ($element eq $current_element) {
	    $key .= $reg;
	    $reg = "";
	    $in_key = 0;
	    print STDERR "Trouve '$key' pour '$current_ana' a $last_line\n" if $debug;
	    if ($element =~ /date$/i) {
		$key = analyze_date($key);
		if ($current_ana eq "diplomatique-date-signature-chancellerie") {
		    $date_signature = $key;
		} elsif ($current_ana eq "crime-date") {
		    $date_crime = $key;
#		    $date_crime_line = $last_line;
		}
	    } else {
		$key = analyze($key);
	    }
	    if (! defined $index{$current_ana}) {
		$index{$current_ana} = {};
	    }
	    ${$index{$current_ana}}{$key} .= href($last_line);
	    $key = "";
	    $current_ana = "";
	    $current_element = "";
	}
    }
}


sub handle_char {
    my ($expat,$string) = @_;

    if ($in_key && ! $reg) {
	$key .= clean_lat1($string);
    }
}

sub handle_init {
    $last_line = "intro";
    $in_key = 0;
    $in_body = 0;
}

sub handle_externent {
  my ($exp, $base, $sys) = @_;	# We don't use pub id

#  print STDERR "$sys\n";
  return "";
}

sub handle_final {
    if ($date_signature && $notaire) {
	print $basename, ".tei; ",UnixDate($date_signature, '%d/%m/%y'), "; ";
	print $notaires{$notaire}, "; ", $notaire, "\n";
    }
    $date_signature = "";
    $notaire = "";
}

$parser->setHandlers(Start => \&handle_start,
		     End   => \&handle_end,
		     Char  => \&handle_char,
		     Init  => \&handle_init,
		     ExternEnt => \&handle_externent,
		     Final => \&handle_final);

$ana = shift;

STDERR->autoflush(1);
print STDERR "Construction de la table pour '$ana'\n";

while (@ARGV) {
  my @m = File::DosGlob::doglob(1,shift);
  @m = sort @m;
  while ($file = shift(@m)) {
      $current_document = $file;
      if ($current_document =~ /(r(B3.-\d+)-.*)\./) {
	  $basename = $1;
	  $docnum = $2;
	  print STDERR $docnum,",";
	  $parser->parsefile($file,ProtocolEncoding => 'ISO-8859-1');
      } 
  }
}

