#!perl -w

use XML::Parser;
use File::DosGlob;
do "subs.pl";

Date_Init("Language=French","DateFormat=nonUS","TZ=MET");

$parser = new XML::Parser(NoExpand => 1);

%index = ();
$debug = 0;
$docnum = 0;
$basename = "";
$current_ana = "";
$reg = "";
$last_line = "";
$date_crime = "";
$date_signature = "";


sub handle_start {
    my ($expat, $element, %atts) = @_;

    if ($element eq "body") {
	$in_body = 1;
    } elsif (! $in_body) {
	return;
    } elsif (defined $atts{ana} &&
	     $atts{ana} =~ /($ana[^ ]*)/o) {
	$current_ana = $1;
	print STDERR "Trouve '$current_ana' a $last_line\n" if $debug;
	$in_key = 1;
	$reg = "";
	$current_element = $element;
	if (defined $atts{reg}) {
	    $reg =  clean_lat1($atts{reg});
	    print STDERR "reg dans $element\n" if (! ($element eq "name" ||
						      $element eq "rs"));
	} elsif (defined $atts{value}) {
	    $reg =  clean_lat1($atts{value});
	}
    } elsif ($element eq "lb") {
	my $n = $atts{n};

	$last_line = $n if $n;
	$key .= " " if $in_key;
    } elsif (! (defined $atts{ana}) &&
	     $element =~ /[Dd]ate$/) {
	print STDERR "\n$basename.tei:";
	print STDERR ($expat->current_line-1);
	print STDERR ":date sans ana\n";
    }

}

sub handle_end {
    my ($expat, $element) = @_;
    if ($in_key) {
	if ($element eq $current_element) {
	    $key .= $reg;
	    $reg = "";
	    $in_key = 0;
	    print STDERR "Trouve '$key' pour '$current_ana' a $last_line\n" if $debug;
	    if ($element =~ /date$/i) {
		$key = analyze_date($key);
		if ($current_ana eq "diplomatique-date-signature-chancellerie") {
		    $date_signature = $key;
		} elsif ($current_ana eq "crime-date") {
		    $date_crime = $key;
#		    $date_crime_line = $last_line;
		}
	    } else {
		$key = analyze($key);
	    }
	    if (! defined $index{$current_ana}) {
		$index{$current_ana} = {};
	    }
	    ${$index{$current_ana}}{$key} .= href($last_line);
	    $key = "";
	    $current_ana = "";
	    $current_element = "";
	}
    }
}


sub handle_char {
    my ($expat,$string) = @_;

    if ($in_key && ! $reg) {
	$key .= $expat->original_string();
    }
}

sub handle_init {
    $last_line = "intro";
    $in_key = 0;
    $in_body = 0;
}

sub handle_externent {
  my ($exp, $base, $sys) = @_;	# We don't use pub id

#  print STDERR "$sys\n";
  return "";
}

sub handle_final {
    if ($date_crime && $date_signature) {
	my $delai = DateCalc($date_crime,$date_signature);
	if ($delai) {
	    $delai = int(Delta_Format($delai,0,"%dt")/30);
	    if ($delai =~ /^\d$/) {
		$delai = "00$delai";
	    } elsif ($delai =~ /^\d\d$/) {
		$delai = "0$delai";
	    }
#	    $delai .= " mois";
	    if (! defined $index{"delai-remission"}) {
		$index{"delai-remission"} = {};
	    }
	    print $basename, ".tei; ",UnixDate($date_signature, '%d/%m/%y'), "; ",$delai,"\n";
#	    $index{"delai-remission"}->{$delai} .= "[" . UnixDate($date_signature,'%d/%m/%Y') . "] " . href($date_crime_line) . "<BR>\n";
	}
    }
    $date_crime = "";
    $date_signature = "";
}

$parser->setHandlers(Start => \&handle_start,
		     End   => \&handle_end,
		     Char  => \&handle_char,
		     Init  => \&handle_init,
		     ExternEnt => \&handle_externent,
		     Final => \&handle_final);

$ana = shift;

STDERR->autoflush(1);
print STDERR "Construction de la table pour '$ana'\n";

while (@ARGV) {
  my @m = File::DosGlob::doglob(1,shift);
  @m = sort @m;
  while ($file = shift(@m)) {
      $current_document = $file;
      if ($current_document =~ /(rB3.-(\d+)-.*)\./) {
	  $basename = $1;
	  $docnum = $2;
	  print STDERR $docnum,",";
	  $parser->parsefile($file,ProtocolEncoding => 'ISO-8859-1');
      } 
  }
}

