#!perl -w

use XML::Parser;
use File::DosGlob;
do "subs.pl";

$parser = new XML::Parser();
$reg = "";
$rkey = "";
$debug = 0;
$docnum = 0;
$basename = "";
$last_line = "";

%index = ();

sub analyze_topo {
    my $name = shift;

    $name =~ s/\s+/ /g;
    $name =~ s/^\s+//g;
    $name =~ s/\s+$//g;
    $name =~ s/\b(s)ainct\b/$1aint/ig;

    if ($name =~ /^de(s?) (.*)$/i) {
	return "$2 (De$1)";
    }
    return $name;
}

sub handle_start {
    my ($expat, $element, %atts) = @_;

    if ($element eq "rs" &&
	defined $atts{type} &&
	$atts{type} eq "toponyme") {
	if ($in_body) {
	    $in_key = 1;
	    if (defined $atts{reg}) {
		$reg = clean_lat1($atts{reg});
	    }
	    if (defined $atts{key}) {
		$rkey = $atts{key};
	    }
	}
    } elsif ($element eq "lb") {
	my $n = $atts{n};

	$last_line = $n if ($n);
	
	$key .= " " if $in_key;
    } elsif ($element eq "body") {
	$in_body = 1;
    } elsif ($element eq "abbr" &&
	     defined $atts{expan} &&
	     $in_key) {
	$expan = $atts{expan};
	$key .= " $expan ";
    }
}

sub handle_end {
    my ($expat, $element, %atts) = @_;
    if ($element eq "rs") {
       if ($in_key) {
	   $in_key--;
	   if ($reg) {
	       print STDERR "reg = $reg\n" if $debug;
	       $key = $reg;
	   }
	   my $k = analyze_topo($key);
	   if ($rkey) {
	       if (defined $reg_table{$rkey}) {
		   $k = $reg_table{$rkey};
		   print STDERR "Clef utilise: $rkey = $k\n" if $debug;
	       } else {
		   $reg_table{$rkey} = $k;
		   print STDERR "Clef definie: $rkey = $k\n" if $debug;
	       }
	   }
#	   print STDERR "Normalizing '$key' into '$k'\n" if (! ($k eq $key));
	   if ($k eq "") {
	       print STDERR "nom vide  $last_line\n";
	   } else {
	       my $ref;
	       if (defined $index{$k} &&
		   $index{$k} =~ ">${docnum}-") {
		   $ref = href($last_line);
	       } else {
		   my $line_ref = href($last_line);
		   $ref = "<b>$line_ref</b>";
	       }
	       if ($element eq "signed") {
		   $ref = "<i>$ref</i>";
	       }
	       $index{$k} .= $ref;
	   }
	       
	   $key = "";
	   $reg = "";
	   $rkey = "";
       }
    } elsif ($element eq "abbr") {
	$expan = "";
    }
}

sub handle_char {
    my ($expat,$string) = @_;

    $key .= clean_lat1($string) if $in_key && ! $expan;
}

sub handle_init {
    $last_line = "intro";
    $in_key = 0;
    $in_body = 0;
}

sub handle_externent {
  my ($exp, $base, $sys) = @_;	# We don't use pub id

#  print STDERR "$sys\n";
  return "";
}

$parser->setHandlers(Start => \&handle_start,
		     End   => \&handle_end,
		     Char  => \&handle_char,
		     Init  => \&handle_init,
		     ExternEnt => \&handle_externent);

STDERR->autoflush(1);
print STDERR "Construction de l'index des toponymes\n";

while (@ARGV) {
  my @m = File::DosGlob::doglob(1,shift);
  @m = sort @m;
  while ($file = shift(@m)) {
      $current_document = $file;
      if ($current_document =~ /(r(B3.-\d+)-.*)\./) {
	  $basename = $1;
	  $docnum = $2;

	  print STDERR $docnum,",";
	  $parser->parsefile($file,ProtocolEncoding => 'ISO-8859-1');
      }
  }
}

print "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n<HTML><HEAD><TITLE>Index</TITLE></HEAD><BODY>";
print "\n";

foreach $k (sort {uc($a) cmp uc($b)} keys(%index)) {
    print "$k, ";
print $index{$k}, "<BR>\n";
}

print "</BODY></HTML>\n";

print STDERR "Ok\n";
